function [Cerr, Rerr, Derr, Rdiffnorm, Ifault]=JHrepair4country(country)
% do the JHrepair for each country
% get data and parameters 
[C, R, D]=GetCoronaData(country);
pars=GetCoronaParameters(country);
len=length(C);
% we work between low and up
up=len
low=pars(1)
% check beforehand
checkmonotonicity(C,'C');
checkmonotonicity(D,'D');
checkmonotonicity(R,'R');
% how many days we look forward  
lookahead=7 % means data of lookahead+1 days are used
deg=2  % the polynomial degree for penalizing smoothness
%
% we now do it on the log of C and D first
%
[logCnew, p, q, Cerr]=myJHrepair(log(max(1,C)), low, up, lookahead, deg);
% check the q and p
qp=[q p]
% the q must sum to one
sum(qp)
% compose new values
Cnew=[C(1:low-1);exp(logCnew)];
% and save as file
SaveCoronaData(country, 'C', Cnew, 'Repaired')
% now same for D
[logDnew,p,q, Derr]=myJHrepair(log(max(1,D)), low, up, lookahead, deg);
%
qp=[q p]
sum(qp)
Dnew=[D(1:low-1);exp(logDnew)];
SaveCoronaData(country, 'D', Dnew, 'Repaired')
%
% To handle R, we have two cases. If no proper R values are there,
% indicated by pars(3)==1, estimate the R values first
% and ignore the given ones, up to an error check
Rdiffnorm=0;
if pars(3)==1
    kdays=14; % for the k-day rule
    % apply the estimator to the repaired C and D values
    pR=estimateRecoveredLinNew(Cnew, Dnew, R, kdays, low, up);
    Rest=pR(kdays+2:end);
    Rdiff=Rest(low:up)-R(low:up);
    Rdiffnorm=norm(Rdiff,2)/(norm(R(low:up),2)*sqrt(up-low+1))
    % and do not repair these new values
    Rnew=Rest;
    Rerr=0;
else
    % treat R the same way as C or D
    [logRnew,p,q, Rerr]=myJHrepair(log(max(1,R)), low, up, lookahead, deg);
    qp=[q p]
    sum(qp)
    Rnew=[R(1:low-1);exp(logRnew)];
end
% display errors
disp(sprintf('Repair errors: estimated: %g, repaired: %g, repair bit: %d',...
    Rdiffnorm, Rerr, pars(3)))
% now check I=C-D-R for positivity, and report failure.
% Such countries are candidates for pars(3)==1, i.e. R estimation.
Inew=Cnew-Rnew-Dnew;
Ifault=0;
if min(Inew)<0
    disp(sprintf('I gets negative for country %s, repair bit: %d',country, pars(3)))
    Ifault=1;
    % pause
end
SaveCoronaData(country, 'R', Rnew, 'Repaired')
% then plot results
figure
hdle=semilogy(low:up,C(low:up),'b')
hold on
semilogy(low:up,Cnew(low:up),'b:',...
    low:up,D(low:up),'r',low:up,Dnew(low:up),'r:',...
    low:up,R(low:up),'g',low:up,Rnew(low:up),'g:',...
    low:up,C(low:up)-R(low:up)-D(low:up),'k',... 
    low:up,Cnew(low:up)-Rnew(low:up)-Dnew(low:up),'k:')
title(sprintf('C, R, D, and I repaired for %s',country))
% save plot as png
saveas(hdle,sprintf('JHrepaired10%s%d',country,len),'png')
% pause
checkmonotonicity(Cnew,'C repaired');
checkmonotonicity(Dnew,'D repaired');
checkmonotonicity(Rnew,'R repaired');

function [Xnew,p,q, err]=myJHrepair(Xin, low, up, lookahead, deg)
% the repair function on a series Xin between low and up,
% with lookahead days and degree deg
% we localize the calling parameters
global loclookahead
global locXin
global locdeg
global locM % number of Xin elements
locdeg=deg;
locXin=Xin(low:up);
loclookahead=lookahead;
locM=length(locXin);
% We optimize over vectors x0=(q,y)
% where the y(j)=x(j)-x(j-1) are nonnegative, j>=2, y(1)=x(1) 
% and the x(j) are what we want
% Starting values 
q=rand(loclookahead,1);
x0=zeros(loclookahead+locM,1);
x0(1:loclookahead)=q;
x0(loclookahead+1)=locXin(1);
% we want  starting values to be close to Xin differences,
% but nonnegative
for j=2:locM
    x0(loclookahead+j)=max(0,locXin(j)-locXin(j-1));
end
options=optimset('MaxFunEvals',100000,'MaxIter',5000);
maxabslocXin=max(1,max(abs(locXin)));
% now optimize with bounds
xopt = lsqnonlin(@myrepairfun,x0,zeros(size(x0)),[ones(loclookahead,1); ones(locM,1)*maxabslocXin],options);
% call the optimization function on the optimum
z=myrepairfun(xopt);
zval=z(1:locM);
err=norm(zval,2)/(norm(locXin,2)*sqrt(length(zval)))
q=xopt(1:loclookahead);
p=q2p(q);
locXinnew=locXin;
locXinnew(1)=xopt(loclookahead+1);
for j=2:locM
    locXinnew(j)=locXinnew(j-1)+xopt(loclookahead+j);
end
Xnew=locXinnew;
% Xnew2=Xnew;
Xnew(locM-2*lookahead:locM)=mysmoother(Xnew(locM-2*lookahead:locM));
% figure
% tt=locM-4*lookahead:locM;
% plot(tt,Xnew(tt),'b',tt,Xnew2(tt),'r')
% legend('First','Second')
% pause


function z=myrepairfun(x);
% this provides the numbers whose squares should be minimized.
global loclookahead
global locXin
global locM 
M=locM;
% the first loclookahead parameters are the q_j
q=x(1:loclookahead);
% the rest are the M unknowns in difference form
% but we need them as y directly, so we have to sum
y=zeros(M,1);
y(1)=x(loclookahead+1);
for j=2:M
    y(j)=y(j-1)+x(loclookahead+j);
end
% we now have to calculate the data error by the model
for j=1:M-loclookahead+1
    z(j)=y(j);
    for k=1:loclookahead
        z(j)=z(j)-q(k)*locXin(j-1+k);       
    end
end
% these were the first M-loclookahead+1 error items.
% We directly take the error of the rest
for j=M-loclookahead+2:M
    z(j)=y(j)-locXin(j); % 
end
% This treats the last values differently than the first.
% A smoother will be required in the calling program
% This makes M terms, now another M for the nonsmoothness penalty
for j=1:M
    z(M+j)=mypenalty(j,y);
end
% then the same for the differences
dy=y(2:end)-y(1:end-1);
for j=1:M-1
    z(2*M+j)=mypenalty(j,dy);
end
% and the final one for the constraint on q
z(3*M)=10000*(1-sum(q));



function val=mypenalty(j,y);
% performs local smoothness test and returns fit quality
% now only for five points and degree 2
val=0;
leny=length(y);
% see if 5 points fit into the range, shift j if not
j=max(3,j);
j=min(leny-2,j);
% This lets ends get a larger weight, but we do not care.
% Now 5 points will fit around j
jmin=j-2;
jmax=j+2;
ym=1.e-6+max(abs(y(jmin:jmax))); % scale factor
% use the fast quadratic error function and scale it
val=sqrt(abs(mypenalty0(y(jmin:jmax))/35))/ym;

function val=mypenalty0(y)
% is zero when the data y(1),...,y(5) can be fitted by a parabola,
% it is 35* of the square of the fitting error, by MAPLE
t1 = y(2) ^ 2;
t3 = y(3) ^ 2;
t5 = y(4) ^ 2;
t7 = y(5) ^ 2;
t9 = y(1) ^ 2;
val = 22 * t1 + 18 * t3 + 22 * t5 + 4 * t7 - 24 * y(2) * y(3) - ...
    12 * y(2) * y(4) - 18 * y(2) * y(1) - 24 * y(3) * y(4) + 6 * y(3) * y(1) + 10 * y(4) * y(1) - ...
    6 * y(5) * y(1) - 18 * y(5) * y(4) + 6 * y(5) * y(3) + 10 * y(5) * y(2) + 4 * t9;


function checkmonotonicity(X, text)
for i=2:length(X)
    if X(i)>=X(i-1)
        continue
    end
    disp(sprintf('%s monotonicity fails between %4d and %4d, values %8d and %8d',text,i,i-1,X(i),X(i-1)))
end

function p=q2p(q)
% we have to calculate the p_i from q_i
% the p_i being probabilities
% the q_i being quantities
D=length(q);
s=1;
p=q;
p(1)=q(1);
for j=2:D
    s=s*(1-q(j-1));
    p(j)=q(j)/s;
end

