function pR=estimateRecoveredLinNew(C, D, kdays, N0, N1)
%
% LINEARIZED version with constraint C-R-D>=0 
%
% Estimation of Recovered from Confirmed and Deaths,
% Estimation of case fatality and case recovery rates
% This program does no smoothing.
% If smoothing is required, do it outside. 
%
% Input:
% 
% C, D : Confirmed and Deaths, same length
% kdays : number of days after which Confirmed 
% are assumed to be Recovered, for k-day rule
% if they do not die in between
% N0, N1 : range of data used
% Data of D between N0-1 and N1 are used
% Data of C between N0-1-kdays and N1-1 are used 
% The linear system has N1-N0+1 equations and kdays unknowns
% 
% Output:
% qopt : survival probability d days after Confirmation
% p : kdays-vector of probabilities p_j to die j days after Confirmation  
% R : estimated Recovered, same length as C and D
% pR: vector of the form [qopt; p; R]
%
if kdays > N1-N0+1
    tdiffk=[N0 N1 kdays]
    error('Not enough equations in estimateRecoveredLinNew')
end 
global A % the matrix and right-hand side 
global rhs
%
dD=zeros(size(D)); % vector for daily deaths
dC=zeros(size(C)); % daily Confirmed
Clen=length(C);
% daily values at day i are stored in i-th position
dD(2:Clen)=D(2:Clen)-D(1:Clen-1); 
dC(2:Clen)=C(2:Clen)-C(1:Clen-1);
% pause
% we use deaths data between N0-1 and N1 
rhs=dD(N0:N1); % D_n-D_{n-1} 
A=zeros(N1-N0+1,kdays); % the data matrix
for j=1:kdays  % form the dC shifts as rows
    A(:,j)=dC(N0-j:N1-j);  
end
B=ones(kdays,1);
q0=zeros(kdays,1); % starting at zero, and lower bounds
options=optimset('MaxFunEvals',50000,'MaxIter',10000,'LargeScale','off');
% perform nonlinear least squares
qopt=lsqlin(A,rhs,[],[],[],[],zeros(kdays,1),ones(kdays,1));
% qopt=lsqlin(A,rhs,[],[],[],[],0,1);
pd=1-sum(qopt); % the survival rate
% now calculate daily Recovered
dR=pd*dC(1:Clen-kdays);
% and insert into full R vector
R=zeros(size(C));
for j=2:Clen-kdays
    R(j+kdays)=R(j-1+kdays)+dR(j-1);
end
R=floor(R);
pR=[pd; qopt; R];

